import { forwardRef, useEffect, useImperativeHandle, useMemo, useState } from "react";
import { Modal, Collapse, message, Empty } from "antd";
import DiffTable from "./DiffTable";
import { useModel, useRequest } from "umi";
import { PushDataModelTable } from "@/api/dataModel";
import { GetAllDesignTables } from "@/api";
import { TableItemType } from "@/type";
import NoData from "@/assets/no-data.png";

export default forwardRef(function SyncModal(props: {onPush: () => void}, ref) {
  const [open, setOpen] = useState(false);
  const { project } = useModel("erModel");
  const { data, loading, run } = useRequest(GetAllDesignTables, {
    manual: true,
  });
  const [tableList, setTableList] = useState<TableItemType[]>(project?.tables);

  useImperativeHandle(ref, () => ({
    open: () => {
      setOpen(true);
      run();
    },
    close: () => setOpen(false),
  }));

  useEffect(() => {
    setTableList(project.tables);
  }, [project.tables]);

  // 已存在的数据表
  const existTableList = useMemo(() => {
    const list: {
      modelTable: TableItemType;
      dataTable: any;
    }[] = [];
    const tables = data?.result?.appBusinessTables || [];
    tableList.forEach((tableItem) => {
      const dataTable = tables.find(
        (item: any) =>
          item.schemaName === tableItem.table.schemaName ||
          item.aliasName === tableItem.table.aliasName
      );
      if (dataTable) {
        list.push({
          modelTable: tableItem,
          dataTable,
        });
      }
    });

    return list;
  }, [tableList, data]);

  const handleUpdateTable = (tableItem: TableItemType) => {
    setTableList(
      tableList.map((item) =>
        item.table.id === tableItem.table.id ? tableItem : item
      )
    );
  }

  const [okLoading, setOkLoading] = useState(false);
  const handleSubmit = async () => {
    try {
      setOkLoading(true);
      await PushDataModelTable(tableList);
      message.success("同步推送完成");
      setOpen(false);
      props.onPush?.();
    } finally {
      setOkLoading(false);
    }
  }

  return (
    <Modal
      title="数据同步推送"
      width={"100%"}
      open={open}
      okText="同步"
      loading={loading}
      style={{
        top: 10,
        padding: 0,
      }}
      bodyProps={{
        style: {
          height: "80vh",
          overflowY: "auto",
        },
      }}
      okButtonProps={{
        loading: okLoading,
        disabled: !tableList.length
      }}
      onCancel={() => setOpen(false)}
      onOk={handleSubmit}
    >
      <div className="text-14px font-bold m-y-12px">
        提示：将当前模型表推送至数据表，一共
        <span className="color-green">{tableList.length}</span>
        张表格数据。
        {existTableList.length ? (
          <>
            其中有<span className="color-#faad14">{existTableList.length}</span>
            张表存在相同的编码或别名，可对比确认后提交。
          </>
        ) : null}
      </div>
      <Collapse
        items={tableList.map((item) => {
          const { table } = item;
          const name = table.langNameList?.find(
            (item) => item.name === "zh-CN"
          )?.value;
          const existTable = existTableList.find(
            (item) => item.modelTable.table.id === table.id
          );
          return {
            key: table.id,
            label: (
              <span
                style={{
                  color: existTable
                    ? "#faad14"
                    : "",
                }}
              >{`${table.schemaName}${name ? `(${name})` : ""}`}</span>
            ),
            children: (
              <DiffTable
                sourceTable={item}
                targetTableId={existTable?.dataTable?.id}
                onChange={handleUpdateTable}
              />
            ),
          };
        })}
      />
      {
        !tableList.length && <Empty image={NoData} description="当前模型表为空，请添加后再进行同步！"/>
      }
    </Modal>
  );
});
